/***********************************************************************
Global variables and functions
************************************************************************/

/*---------------------------------------------------------------------
     Definition area. Write pin definition here.
----------------------------------------------------------------------*/
#define SS_PIN    4           // assign D4 pin to slave select pin for 7SEG_LED.
#define SCK_PIN   7           // assign D7 pin to serial clock pin for 7SEG_LED.
#define  MOSI_PIN 8           // assign D8 pin to serial data out pin for 7SEG_LED.

#define LED1_PIN  13          // assign D13 pin for LED1
#define LED2_PIN  12          // assign D12 pin for LED2
#define LED3_PIN  11          // assign D11 pin for LED3
#define LED4_PIN  10          // assign D10 pin for LED4

#define ex_swPin A1           // assign A1 pin to ex_swPin for external SW.
#define swPin 26              // assign D26 pin to swPin for SW_USER.


const byte LED_SEGMENT[] =
{   //ABCDEFGH  negative logic 0:ON/1:OFF
  0b00000011,   // segment data for 0
  0b10011111,   // segment data for 1
  0b00100101,   // segment data for 2
  0b00001101,   // segment data for 3
  0b10011001,   // segment data for 4
  0b01001001,   // segment data for 5
  0b01000001,   // segment data for 6
  0b00011111,   // segment data for 7
  0b00000001,   // segment data for 8
  0b00001001,   // segment data for 9
  0b11111111    // space
};

const byte LED_DIGIT[] =
{
  0x8F,   // select 1st digit from left
  0x4F,   // select 2nd digit from left
  0x2F,   // select 3rd digit from left
  0x1F    // select 4th digit from left
};

const byte LED_DATA[] =
{
  0xFF,     // output data for 0
  0xF7,     // output data for 1
  0xFB,     // output data for 2
  0xFD,     // output data for 3
  0xFE,     // output data for 4
  0xF6,     // output data for 5
  0xFA,     // output data for 6
  0xFC,     // output data for 7
  0xF4,     // output data for 8
  0xF8,     // output data for 9
};

int old_time = 0x0000;          // previous time(milli sec.)

char mini_data = 0;             // minute data 0:1:3:7:F:1F:3F:7F:FF
char sec_data = 0;              // second count data(0 to 59)
char precount1 = 0;             // prescaler for 100milli seconds.(25)
char precount2 = 0;             // prescaler(10) for 1second
char digi_sel = 0;              // digit select

char prescale4sw = 0;           // prescaler for SW check( up to 5:20milli sec.)
int sw_data = 0xFFFF;           // SW data image(history of 20 milli sec. interval)
int time_mode = 0;              // o:stop/1:run

void setup() {
  // put your setup code here, to run once:
    pinMode(SS_PIN, OUTPUT);      // set D4pin to output mode
    pinMode(SCK_PIN, OUTPUT);     // set D7pin to output mode
    pinMode(MOSI_PIN, OUTPUT);    // set D8pin to output mode
    pinMode(ex_swPin, INPUT);     // set A1pin to input mode
    pinMode(swPin, INPUT);        // set D26pin to input mode

    pinMode(LED1_PIN, OUTPUT);    // set D13pin to output mode
    pinMode(LED2_PIN, OUTPUT);    // set D12pin to output mode
    pinMode(LED3_PIN, OUTPUT);    // set D11pin to output mode
    pinMode(LED4_PIN, OUTPUT);    // set D10pin to output mode

    digitalWrite( SCK_PIN, LOW ); // set SCK signal to LOW

}

void loop() {
  // put your main code here, to run repeatedly:

    int time_work;                      // present time buffer
    char com_sel;                       // common select signal
    char seg_data;                      // segment data
    char sw_work;                       // work for switch check
    byte dt_wk;                         // time data work
    byte mdata;                         // minute data for LED1-4

/*--------------------------------------
    wait for 4milli seconds interval.
----------------------------------------*/

    time_work = ( int )( millis() & 0x0FFC ); // read milli sec data

    if ( old_time != time_work )        // check timing of change LED
    {

/*--------------------------------------
    every 4milli seconds timing
    change display digit.
----------------------------------------*/

        old_time = time_work;           // change current time data
        time_work &= 0x00C;             // get comsel data(= 4milli second unit)
    
        switch ( time_work )
        {
            case 0x00:                  // 10second digit timing
                dt_wk = ( sec_data / 10 );      // get 10second digit data
                seg_data = LED_SEGMENT[dt_wk];  // set segment of 10second digit data
                com_sel = LED_DIGIT[1];         // set 10second digit
                break;

            case 0x04:                  // second digit timing
                dt_wk = ( sec_data % 10 );      // get second digit data
                seg_data = LED_SEGMENT[dt_wk];  // set segment of second digit data
                seg_data &= 0xFE;               // set point
                com_sel = LED_DIGIT[2];         // set second digit
                break;

            case 0x08:                  // 100milli second digit timing
                dt_wk = precount2;              // get 100milli second digit data
                seg_data = LED_SEGMENT[dt_wk];  // set segment of 100milli second digit data
                com_sel = LED_DIGIT[3];         // set 100milli second digit
                break;

            default:                    // minute digit timing
                dt_wk = mini_data;              // get minute display data
                seg_data = 0xFF;                // set segment clear data
                com_sel = LED_DIGIT[0];         // select minute digit

                mdata = LED_DATA[dt_wk];        // get data for minute LED

                digitalWrite( LED4_PIN, (mdata & 1 ));
                mdata >>= 1;
                digitalWrite( LED3_PIN, (mdata & 1 ));
                mdata >>= 1;
                digitalWrite( LED2_PIN, (mdata & 1 ));
                mdata >>= 1;
                digitalWrite( LED1_PIN, (mdata & 1 ));

        }

        digitalWrite( SS_PIN, LOW );      // turn on SS signal(low level)

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, seg_data); // segment data out

        shiftOut (MOSI_PIN, SCK_PIN, LSBFIRST, com_sel);  // digit data out

        digitalWrite( SS_PIN, HIGH ); // turn off SS signal(rise up)

/*--------------------------------------
    every 4milli seconds timing
    if count enable, then count time up.
----------------------------------------*/

        if (( 25 <= ++precount1 ) && ( 1 == time_mode ))
        {                               // 100 milli seconds passing
            precount1 = 0;              // clear 100milli second counter
            if (  10 <= ++precount2 )   // count for seconds
            {                           // one second passing
                precount2 = 0;          // clear one second counter
                if ( 60 <= ++sec_data )
                {                       // one minute passing
                    sec_data = 0;
                    if ( 9 == mini_data )
                    {                   // over flow
                        mini_data = 0;  // clear minute data
                    }
                    else
                    {                   // change minute data
                        ++mini_data;   // count minute data
                    }
                }
            }
        }

/*--------------------------------------
    switch data check timing
----------------------------------------*/

        if ( ++prescale4sw >= 5 )       // check SW check timing or not
        {                               // 20milli seconds passing
            prescale4sw = 0x00;         // clear timing counter
            sw_data <<= 1;              // shift olddata left
            sw_data += ( digitalRead(ex_swPin)&digitalRead(swPin) ); // read sw data
            sw_work = (sw_data & 0x06); // extract check timing data
            if ( 0x04 == sw_work )
            {                           // sw is pushed
                time_mode ^= 0x01;      // change count mode flag
            }
        }
    
    }


}
